/* 
 * wsdlpull - A C++ parser  for Wsdl  (Web services description language)
 * Copyright (C) 2005-2007 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _OPERATIONH
#define  _OPERATIONH

#include <xmlpull/Qname.h>
#include <wsdlparser/Wsdl.h>
#include <wsdlparser/WsdlException.h>
#include <wsdlparser/WsdlElement.h>
#include <wsdlparser/Message.h>

class PortType;
using namespace Wsdl;
//class for Wsdl operation element
class Operation:public WsdlElement
{
 public:
  typedef std::vector<Operation*>::iterator OpIterator;
  typedef std::vector<Operation*>::const_iterator cOpIterator;

  Operation(WsdlParser& w,PortType * pt);
  ~Operation();
  /** @name Various Getter methods*/
  //@{

  /**
   * getMessage
   * @param mesage type ,Input,Output or Fault
   * @return pointer to the message
   */
  const Message *getMessage(Wsdl::MessageType type) const;
  /**
   * getOptype
   * @return type of the operation in,in-out,out,out-in
   */
  Optype getType() const;
  
  /**
   * portType
   * @return The porttype to which this operation belongs
   */
  const PortType* portType()const;

  //@}/** @name Various Setter methods*///@{
  void setMessage(const Message * message, Wsdl::MessageType type);
  //@}
  void print(std::ostream & out);

 private:
  const Message *inMessage_, *outMessage_, *faultMessage_;
  Optype type_;
  PortType * pt_;
};



inline
Operation::Operation(WsdlParser& w,PortType * p)
  :WsdlElement(w),
     pt_(p)
{
  type_ = NONE;
  inMessage_ = outMessage_ = faultMessage_ = 0;
}

inline
Operation::~Operation()
{
}

inline
const PortType* 
Operation::portType()const
{
  return pt_;
}

inline
const Message *
Operation::getMessage(Wsdl::MessageType type) const     
{
  if (type == Input)
    return inMessage_;

  else if (type == Output)
    return outMessage_;

  else if (type == Fault)
    return faultMessage_;

  else
    return 0;
}

inline  
Optype
Operation::getType() const                        
{
  return type_;
}


inline
void 
Operation::setMessage(const Message * message,
		      Wsdl::MessageType type)
{
  if (message == 0)
    throw WsdlException("Invalid message name");
  if (type == Input)
    {
      inMessage_ = message;
      if (type_ == NONE)
	type_ = IN;

      else if (type_ == OUT)
	type_ = OUT_IN;

      else
	type_ = NONE;
    }
  else if (type == Output)
    {
      outMessage_ = message;
      if (type_ == NONE)
	type_ = OUT;

      else if (type_ == IN)
	type_ = IN_OUT;

      else
	type_ = NONE;
    } else if (type == Fault)
      faultMessage_ = message;
}

inline
void 
Operation::print(std::ostream & out)
{
  out << id_ << XmlUtils::dbsp << name_ << std::endl;
  out << type_ << std::endl;
  out << inMessage_ << XmlUtils::dbsp << outMessage_ << XmlUtils::dbsp <<
    faultMessage_ << std::endl;
  out << XmlUtils::blk;
}
#endif                                            /*  */
