/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2005-2007 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef  WITH_CURL
#include <curl/curl.h>
#endif 
#include <wsdlparser/WsdlInvoker.h>

extern "C" {
  size_t storeResults(void * buf,size_t sz,size_t nmemb,void* userdata);
}
static char* results_ = 0;

WsdlInvoker::WsdlInvoker()
  :wParser_(0),
   xmlStream_(0),
   soapstr_(0),
   status_(false),
   serializeMode_(false),
   verbose_(false),
   op_(0),
   n_(0)
{
}

WsdlInvoker::WsdlInvoker(const std::string & url)
  :wParser_(0),
   xmlStream_(0),
   status_(false),
   serializeMode_(false),
   verbose_(false),
   op_(0),
   n_(0)
{
  parseWsdl(url);
}

void
WsdlInvoker::parseWsdl(const std::string & url)
{
  try{
    wParser_ = new WsdlParser(url,logger_);
    if (wParser_){
      //parse the web service
      while (wParser_->getNextElement () != WsdlParser::END);
      if (wParser_->status()){
      
	status_=true;
      }
    }
    if (status_){
    
      PortType::cPortTypeIterator p1,p2;
      wParser_->getPortTypes(p1,p2);
      int i=0;
      Soap* soap=static_cast<Soap*> (wParser_->getExtensibilityHandler(Soap::soapBindingUri));
      while(p1!=p2){
    
	Operation::cOpIterator op1,op2;
	(*p1)->getOperations(op1,op2);
	const Binding *bn = (*p1)->binding(Soap::soapBindingUri);
	if (!bn){
	  p1++;
	  continue;
	}
	int soap_binding_elem =soap->getElementName (bn->getBindingInfo ());
	//check if the port type has a soap binding
	if (soap_binding_elem == 0){
	  p1++;
	  continue;
	}

	while(op1!=op2){
	
	  opMap_[(*op1)->getName()]=*op1;
	  op1++; 
	  i++;
	}
	p1++;
      }
    }
  }
  catch (WsdlException we)
    {
      logger_<<"An Exception occurred ...@"<<we.line   
	     <<":"<<we.col<<std::endl;
      logger_<<we.description<<std::endl;
      status_ =false;
    }
  catch (SchemaParserException spe)
    {
      logger_<<"An Exception occurred ...@"<<spe.line
	     <<":"<<spe.col<<std::endl;
      logger_<<spe.description<<std::endl;
      status_ =false;
    }
  catch (XmlPullParserException xpe)
    {
      logger_<<"An Exception occurred ...@"<<xpe.line
	     <<":"<<xpe.col<<std::endl;
      logger_<<xpe.description<<std::endl;
      status_ =false;
    }
}

int
WsdlInvoker::getOperations(std::vector<std::string> & operations)
{
  int i = 0;
  for(
  std::map<std::string,const Operation*>::iterator it =
    opMap_.begin();
  it != opMap_.end();
  it++,i++){

    operations.push_back(it->first);
  }
  return i;
}

std::string
WsdlInvoker::getOpDocumentaion(const std::string & n)
{
  
  std::map<std::string,const Operation*>::iterator it =
    opMap_.find(n);
  
  if (it != opMap_.end()){
    
    return  it->second->getDocumentation();
  }
  return "";
}

bool
WsdlInvoker::setOperation(const std::string & opname)
{
  reset();
  
  std::map<std::string,const Operation*>::iterator it =
    opMap_.find(opname);

  if (it != opMap_.end()){
    
    op_ = it->second;

    const Binding * bnSoap = op_->portType()->binding(Soap::soapBindingUri);
    Soap* soap=
      static_cast<Soap*> (wParser_->getExtensibilityHandler(Soap::soapBindingUri));    

    ///get various soap properties
    soap->getServiceLocation (bnSoap->getServiceExtId (),location_);
    style_ = soap->getStyle();

    //get the soap:operation's SOAPAction and style
    const int *bindings = 0;
    int opIndex  = op_->portType()->getOperationIndex(op_->getName());
    bnSoap->getOpBinding (opIndex, bindings);
    int soapOpBindingId = bindings[0];
    //operation's style over rides 
    soap->getSoapOperationInfo (soapOpBindingId, action_, style_);

    //get the soap:body namespace and use attributes
    bnSoap->getInputBinding(opIndex,bindings);
    int  soapInputBinding = bindings[0];
    soap->getSoapBodyInfo(soapInputBinding,nsp_,use_);
    
    if (nsp_.empty()){

      nsp_ = wParser_->getNamespace();
    }
    serialize();
    return true;
  }
  else{
    return false;
  }
}

//this method extracts the  atomic types needed for the web service
void
WsdlInvoker::serialize()
{
  const Message * m = op_->getMessage(Wsdl::Input);
  
  for (int i = 0 ;i<m->getNumParts();i++){
      
    Part::PartRefType prt = m->getPartRefType(i);
    const Part * p = m->getMessagePart(i);
    const SchemaParser * sParser = wParser_->getSchemaParser(p->schemaId());
    
    if (prt == Part::Elem){
      
      const Element * e = p->element();
      serializeType((Schema::Type)e->getType(),e->getName(),sParser,1,1);
      
    }
    else{
      
      serializeType((Schema::Type)p->type(),p->name(),sParser,1,1);
    }
  }
}

void
WsdlInvoker::serializeType(Schema::Type typeId,
			   const std::string &tag,
			   const SchemaParser * sParser,
			   int min,
			   int max)
{

  const XSDType * pType = sParser->getType(typeId);
  if ( pType== 0 ||
       pType->isSimple() ||
       pType->getContentModel() == Schema::Simple){
    
    if (serializeMode_ == false){
      
      Parameter p(typeId,tag,min,max,sParser);
      elems_.push_back(p);
    }
    else{
      //generate the xml
      serializeParam(n_++,tag,sParser);
    }
  }
  else{

    if (serializeMode_){
      
      if (style_ == Soap::DOC){
    
	xmlStream_->setPrefix("s",sParser->getNamespace());
	xmlStream_->startTag(sParser->getNamespace(),tag);
      }
      else{
	xmlStream_->startTag("",tag);
      }
    }
    
    const ComplexType * ct =
      static_cast<const ComplexType*>(pType);
    
    //complex types handling
    if (ct->getNumAttributes() > 0) {
      
      for (int i = 0; i < ct->getNumAttributes(); i++) {
	  
	const Attribute*at = ct->getAttribute(i);
	/*
	 * Check for the correctness of each attribute
	 */
	if (at->isRequired()){
	
	  if (serializeMode_ == false){

	    Parameter p((Schema::Type)at->getType(),at->getName(),elems_.size(),0,sParser);
	    attribs_.push_back(p);
	  }
	  else{
	    //generate the xml
	    //TODO
	  }  
	}
	else
	  continue;
      }
    }
  
    if (ct->getContentModel() == Schema::Simple) {

      if (serializeMode_ == false){
      
	Parameter p((Schema::Type)ct->getContentType(),tag,min,max,sParser);
	elems_.push_back(p);
	
      }
      else{
	//generate the xml
	serializeParam(n_++,tag,sParser);
      }
    }
    else{
          
      ContentModel* cm=ct->getContents();
      if(cm)
	serializeContentModel(cm,sParser);
    }

    if (serializeMode_){

      if (style_ == Soap::DOC){
    
	xmlStream_->endTag(sParser->getNamespace(),tag);
      }
      else{
	xmlStream_->endTag("",tag);
      }
    }
  }
}

void
WsdlInvoker::serializeContentModel(ContentModel *cm,
				   const SchemaParser *sParser)
{
  
  ContentModel::ContentsIterator cit_b=cm->begin();
  ContentModel::ContentsIterator cit_e=cm->end();
  ContentModel::ContentsIterator ci=cit_b;

  switch (cm->getCompositor())
    {
    case Schema::All:
    case Schema::Sequence:
    case Schema::Choice:
      {
	// a simple logic to start with
	// not taking care of all,choice ,sequence as of now
	
	for (ci=cit_b;ci!=cit_e;ci++){
	  
	  if(ci->second==ContentModel::Particle &&
	     ci->first.e->max() > 0){
	    
	    serializeType((Schema::Type)ci->first.e->getType(),
			  ci->first.e->getName(),
			  sParser,
			  ci->first.e->min(),
			  ci->first.e->max());
	  }
	  else if (ci->second==ContentModel::Container) {
	    
	    //nested xsd:sequence inside choice..nested content models
	    serializeContentModel(ci->first.c,
				  sParser);
	      
	  }
	  else if (ci->second==ContentModel::ParticleGroup){
	    
	    //xsd:group inside 
	    serializeContentModel(ci->first.g->getContents(),
				  sParser);
	  }
	}
	break;
      }
    }
}


void
WsdlInvoker::serializeParam(int n,const std::string & tag,
			    const SchemaParser * sParser)
{

  std::string t=tag;
  if (tag=="*")
    t="item";
    
  for (int i = 0 ;i<elems_[n].n_;i++){

    if (style_ == Soap::DOC){
    
      xmlStream_->setPrefix("s",sParser->getNamespace());
      xmlStream_->startTag(sParser->getNamespace(),t);
    }
    else{
      
	xmlStream_->startTag("",t);

      //xsi::type is needed for many SOAP servers
      if (sParser->isBasicType(elems_[n].type_)){

	xmlStream_->attribute(Schema::SchemaInstaceUri,
			      "type",
			      "xsd:"+sParser->getTypeName(elems_[n].type_));
      }
    }
    
    xmlStream_->text(elems_[n].data_[i]);
	
    if (style_ == Soap::DOC){
    
      xmlStream_->endTag(sParser->getNamespace(),t);
    }
    else{

      xmlStream_->endTag("",t);
    }
  }
}


bool
WsdlInvoker::setInputValue(const int param,void** values,unsigned int occurs)
{

  if (occurs < elems_[param].min_ ||
      occurs > elems_[param].max_)
    return false;

  SchemaValidator *sv = new SchemaValidator (elems_[param].sParser_);
  for (unsigned int i = 0 ;i < occurs ;i++){
      
    TypeContainer * tc = sv->validate(values[i],
				      elems_[param].type_);
    if (!tc->isValueValid()){
	
      return false;
    }
    std::ostringstream oss;
    tc->print(oss);
    elems_[param].data_.push_back(oss.str());
    delete tc;
  }
  delete sv;
  
  elems_[param].n_ = occurs;
  return true;
}

bool
WsdlInvoker::setInputValue(const int param,std::vector<std::string> values)
{


  if (values.size() < elems_[param].min_ ||
      values.size() > elems_[param].max_)
    return false;

  SchemaValidator *sv = new SchemaValidator (elems_[param].sParser_);
  
  for (size_t i = 0 ;i < values.size() ;i++){
      
    TypeContainer * tc = sv->validate(values[i],
				      elems_[param].type_);
    if (!tc->isValueValid()){
	
      return false;
    }
    elems_[param].data_.push_back(values[i]);
    delete tc;
  }
  delete sv;
  
  elems_[param].n_ = values.size();
  return true;
}

bool
WsdlInvoker::setInputValue(const int param,std::string val)
{

  const SchemaParser* sParser = elems_[param].sParser_;
  SchemaValidator *sv = new SchemaValidator (sParser);
  Schema::Type t = elems_[param].type_;
  const XSDType * pType = sParser->getType(t);
  if (pType && !pType->isSimple()){

    if (pType->getContentModel() != Schema::Simple)
      return false;

    const ComplexType * ct = static_cast<const ComplexType*>(pType);
    t = (Schema::Type)ct->getContentType();
  }
  
  TypeContainer * tc = sv->validate(val,t);
  if (!(tc && tc->isValueValid())){
	
    return false;
  }
  if (elems_[param].data_.size() == 0)
    elems_[param].data_.push_back(val);
  else
    elems_[param].data_[0]=val;

  delete tc;
    
  delete sv;
  
  elems_[param].n_ = 1;
  return true;
}



bool
WsdlInvoker::setInputValue(const int param,void* val)
{

  const SchemaParser*  sParser = elems_[param].sParser_;
  SchemaValidator *sv = new SchemaValidator (sParser);
  Schema::Type t = elems_[param].type_;
  const XSDType * pType = sParser->getType(t);
  if (pType && !pType->isSimple()){

    if (pType->getContentModel() != Schema::Simple)
      return false;

    const ComplexType * ct = static_cast<const ComplexType*>(pType);
    t = (Schema::Type)ct->getContentType();
  }

  TypeContainer * tc = sv->validate(val,t);
  if (!(tc && tc->isValueValid())){
	
    return false;
  }
  std::ostringstream oss;
  tc->print(oss);
  if (elems_[param].data_.size() == 0)
    elems_[param].data_.push_back(oss.str());
  else
    elems_[param].data_[0]=oss.str();
  delete tc;
  delete sv;
  elems_[param].n_ = 1;
  return true;
}

bool
WsdlInvoker::setValue(const std::string & param,void* val)
{
  for (size_t s = 0;s<elems_.size();s++){
    
    if (elems_[s].tag_ == param)
      return setInputValue(s,val);
  }
  return false;
}

bool
WsdlInvoker::setValue(const std::string & param,void** values,unsigned int occur)
{

  for (size_t s = 0;s<elems_.size();s++){
    
    if (elems_[s].tag_ == param)
      return setInputValue(s,values,occur);
  }
  return false;
}
 
bool
WsdlInvoker::setValue(const std::string & param,std::string val)
{
  for (size_t s = 0;s<elems_.size();s++){
    
    if (elems_[s].tag_ == param)
      return setInputValue(s,val);
  }
  return false;
}

bool
WsdlInvoker::setValue(const std::string & param,std::vector<std::string> values)
{
  for (size_t s = 0;s<elems_.size();s++){
    
    if (elems_[s].tag_ == param)
      return setInputValue(s,values);
  }
  return false;
}


bool
WsdlInvoker::invoke()
{
  n_ = 0;

  if (xmlStream_){
    
    delete xmlStream_;
  }
  if (soapstr_){
    
    delete soapstr_;
  }
  if (results_){
    delete results_;
    results_ = 0;
  }

  for (size_t x = 0;x<outputs_.size();x++)
    delete outputs_[x].second;

  outputs_.clear();

  soapstr_ = new std::ostringstream();
  xmlStream_ = new XmlSerializer(*soapstr_);
  
  serializeMode_ = true;
  xmlStream_->setPrefix("ns",nsp_);
  xmlStream_->startDocument("UTF-8",false);
  xmlStream_->setPrefix("SOAP-ENV",Soap::soapEnvUri);
  xmlStream_->setPrefix("SOAP-ENC",Soap::soapEncUri);
  xmlStream_->setPrefix("xsd",Schema::SchemaUri);
  xmlStream_->setPrefix("xsi",Schema::SchemaInstaceUri);
  xmlStream_->startTag(Soap::soapEnvUri,"Envelope");
  
  if (style_ == Soap::RPC) {
    
    xmlStream_->attribute(Soap::soapEnvUri,
			  "encodingStyle",
			  Soap::soapEncUri);
  }

  xmlStream_->startTag(Soap::soapEnvUri,"Body");
  if (style_ == Soap::RPC){
    
    xmlStream_->startTag(nsp_,op_->getName());
  }

  serialize();
  if (style_ == Soap::RPC){
    xmlStream_->endTag(nsp_,op_->getName());
  }

  xmlStream_->endTag(Soap::soapEnvUri,"Body");
  xmlStream_->endTag(Soap::soapEnvUri,"Envelope");
  xmlStream_->flush();
  post();
  n_ = 0;
  //test
  //status_ = true;
  //processResults();
  // return status_;
  //test
  if (results_){
    processResults();
    if (status_)
      return true;
  }
  else{
    
    logger_<<"Couldnt connect to "<<location_;
  }
    
  return false;
}

int
WsdlInvoker::getNextInput(std::string & param ,Schema::Type & type,int & min,int & max)
{
  if (n_ < elems_.size()){
    
    param = elems_[n_].tag_;
    type = elems_[n_].type_;
    min =  elems_[n_].min_;
    max =  elems_[n_].max_;
    return n_++;
  }
  else{
    return -1;
  }
}

void
WsdlInvoker::processResults()
{
  try{

    const Message* m = op_->getMessage(Wsdl::Output);
    std::istringstream respstr(results_);
    //std::ifstream respstr("r.log");//test
    XmlPullParser* xpp = new XmlPullParser(respstr);
    xpp->setFeature (XmlPullParser::FEATURE_PROCESS_NAMESPACES, true);
    xpp->require (XmlPullParser::START_DOCUMENT, "", "");

    while (xpp->getEventType () != XmlPullParser::END_DOCUMENT) {

      if (xpp->getEventType () == XmlPullParser::END_DOCUMENT)
	break;
      
      if (xpp->getEventType () == XmlPullParser::END_TAG &&
	  xpp->getName() == "Envelope" &&
	  xpp->getNamespace() ==  Soap::soapEnvUri)
	break;
	
      if (xpp->getEventType () != XmlPullParser::START_TAG){
	xpp->nextToken ();
	continue;
      }
      
      xpp->nextTag ();
      Qname elemName (xpp->getName ());
      elemName.setNamespace(xpp->getNamespace());
      
      if (elemName.getNamespace() == Soap::soapEnvUri)
	continue; //ignore soap:Body and soap:Envelope

      if (style_ == Soap::RPC && use_==Soap::ENCODED){
	  
	if (elemName.getLocalName()== op_->getName()+"Response") {

	  //operation's name followed by 'Response' must be the containing element
	  xpp->nextTag ();
	  //look for xsi:type
	  Qname typ(xpp->getAttributeValue(Schema::SchemaInstaceUri, "type"));
	  typ.setNamespace(xpp->getNamespace(typ.getPrefix()));
	  const SchemaParser * sParser = 0;
	  int typeId = 0;
	  
	  if (!(typ.getNamespace() == Soap::soapEncUri &&
		typ.getLocalName() == "Array"))//for soap array just use the part's type info
	    sParser= wParser_->getSchemaParser(typ.getNamespace());
	  
	  if (sParser){
	    
	    typeId = (const_cast<SchemaParser*>(sParser))->getTypeId(typ);
	  }
	  else{
	    //if xsi:type doesnt give a clue then see if the part name matches
	    const Part * p = m->getMessagePart(xpp->getName ());
	    if (p){
	      sParser = wParser_->getSchemaParser(p->schemaId());
	      typeId = p->type();
	    }
	  }
	  if (sParser && typeId !=0){  
	    
	    SchemaValidator * sv= new SchemaValidator(sParser);	  
	    std::string tag = xpp->getName();
	    TypeContainer * t = sv->validate (xpp, typeId);
	    outputs_.push_back(std::pair<std::string,TypeContainer*>(tag,t));
	  }
	  else{

	    status_ = false;
	    logger_<<"Unknown Element"<<elemName<<std::endl;
	    return;
	  }
	}
      }
      else{
	//doc/literal has ref type element in the part
	const SchemaParser * sParser =
	  wParser_->getSchemaParser(elemName.getNamespace());
	if (!sParser){
	  
	  status_ = false;
	  logger_<<"Unknown Element"<<elemName<<std::endl;
	  return;
	}
	SchemaValidator * sv= new SchemaValidator(sParser);	  
	  
	const Element * e = sParser->getElement (elemName);
	if(e){
	  int typeId = e->getType () ;
	  TypeContainer * t = sv->validate (xpp, typeId);
	  std::pair<std::string,TypeContainer*> pr(elemName.getLocalName(),t);
	  outputs_.push_back(pr);
	}
	else{
	  std::cerr<<"Unkown element "<<elemName.getLocalName()<<std::endl;
	}
      }
    }
    delete xpp;
    status_ = true;
  }
  catch (WsdlException we)
    {
      logger_<<"An Exception occurred ...@"<<we.line   
	     <<":"<<we.col<<std::endl;
      logger_<<we.description<<std::endl;
      status_ =false;
    }
  catch (SchemaParserException spe)
    {
      logger_<<"An Exception occurred ...@"<<spe.line
	     <<":"<<spe.col<<std::endl;
      logger_<<spe.description<<std::endl;
      status_ =false;
    }
  catch (XmlPullParserException xpe)
    {
      logger_<<"An Exception occurred ...@"<<xpe.line
	     <<":"<<xpe.col<<std::endl;
      logger_<<xpe.description<<std::endl;
      status_ =false;
    }
  return;
}

WsdlInvoker::~WsdlInvoker()
{
  delete wParser_;
  if (xmlStream_){
    
    delete xmlStream_;
  }
  if (soapstr_){
    
    delete soapstr_;
  }
  reset();
}

void
WsdlInvoker::reset()
{
  n_ = 0;
  elems_.clear();
  attribs_.clear();
  for (size_t x = 0;x<outputs_.size();x++)
    delete outputs_[x].second;

  outputs_.clear();
}

bool 
WsdlInvoker::getNextOutput(std::string & name,TypeContainer * & tc)
{
  if (status_ && n_ < outputs_.size()){
    
    name = outputs_[n_].first;
    tc = outputs_[n_].second;
    n_++;
    return true;
  }
  return false;
}


TypeContainer*
WsdlInvoker::getOutput(const std::string  & name)
{
  for (unsigned int i = 0 ;status_ && i <outputs_.size();i++){

    if ( name == outputs_[n_].first)
      return outputs_[n_].second;
  }
  return 0;
}

void * 
WsdlInvoker::getValue(const std::string  & name ,Schema::Type & t)
{
  for (unsigned int i = 0 ;status_ && i <outputs_.size();i++){

    if (outputs_[n_].second!=0){

      void * tmp= outputs_[n_].second->getValue(name,t);
      if (tmp)
	return tmp;
    }
  }
  return 0;
}


size_t
storeResults(void * buf,size_t sz,size_t nmemb,void* userdata)
{
  int *bufsize= (int*)userdata;
  if (results_ == 0){
    
    results_ = (char*)malloc(sizeof(char) * sz * nmemb);
  }
  else{
    results_ = (char*) realloc(results_,sizeof(char) * sz * nmemb+ (*bufsize));
  }
  memcpy (results_+(*bufsize),buf,sz*nmemb);
  *bufsize+=sz*nmemb;
  return sz*nmemb;
}

void
WsdlInvoker::post()
{
  const std::string  postData = soapstr_->str();
  if(verbose_){
    
    std::ofstream ofs("request.log",std::ios::app);
    ofs<<postData;
    ofs<<std::endl;
    ofs.flush();
  }
    
#ifdef WITH_CURL
  CURL * ctx=0;
  CURLcode res;
  curl_global_init( CURL_GLOBAL_ALL ) ;
  ctx=curl_easy_init();
  int bufsize = 0;
  if (!ctx)
    return ;
  curl_easy_setopt( ctx , CURLOPT_URL,  location_.c_str()) ;

  curl_easy_setopt( ctx , CURLOPT_NOPROGRESS , 1 ) ;

  if (verbose_) {
    curl_easy_setopt( ctx , CURLOPT_VERBOSE,1);
    curl_easy_setopt( ctx , CURLOPT_NOPROGRESS , 0 ) ;
  }

  curl_easy_setopt( ctx , CURLOPT_POST , 1 );
  curl_easy_setopt( ctx , CURLOPT_POSTFIELDS , postData.c_str()) ;
  curl_slist* responseHeaders = NULL ;
  std::string tmp="SOAPAction: ";
  tmp.push_back('"');
  tmp+=action_;
  tmp.push_back('"');
  responseHeaders = curl_slist_append( responseHeaders , tmp.c_str());
  responseHeaders = curl_slist_append( responseHeaders ,"Content-Type: text/xml; charset=UTF-8");
  responseHeaders = curl_slist_append( responseHeaders ,"Accept: text/xml;");
  curl_easy_setopt( ctx , CURLOPT_HTTPHEADER , responseHeaders ) ;
  tmp = "wsdlpull/";
#ifdef HAVE_CONFIG_H
  tmp+=VERSION;
#endif
  curl_easy_setopt( ctx,CURLOPT_USERAGENT,tmp.c_str());
  curl_easy_setopt( ctx,CURLOPT_POSTFIELDSIZE,postData.length());
  
  if (XmlUtils::proxy){
    curl_easy_setopt(ctx,CURLOPT_PROXY,XmlUtils::proxy_host.c_str());
    tmp=XmlUtils::proxy_user+":"+XmlUtils::proxy_pass;
    curl_easy_setopt(ctx,CURLOPT_PROXYUSERPWD,tmp.c_str());
  }
  curl_easy_setopt( ctx ,CURLOPT_WRITEDATA ,&bufsize) ;
  curl_easy_setopt( ctx ,CURLOPT_WRITEFUNCTION,storeResults) ;
  
  // std::logger_ << "- - - BEGIN: response - - -" << std::endl ;
  res=curl_easy_perform(ctx);
  //  std::logger_ << "- - - END: response - - -" << std::endl ;

  curl_slist_free_all( responseHeaders ) ;
  curl_easy_cleanup( ctx ) ;
  curl_global_cleanup() ;
  if(verbose_ && results_){
    
    std::ofstream ofs("response.log",std::ios::app);
    ofs<<results_;
    ofs<<std::endl;
    ofs.flush();
  }
  
#endif
}
