/* 
 * wsdlpull- A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2005-2007 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _WIN32
#include <termios.h>
#include <unistd.h>
#endif
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef WITH_CURL
#include <curl/curl.h>
#endif
#include <fstream>
#include <xmlpull/XmlUtils.h>

//put all I/O and std::string manip  utiliy functions here

bool XmlUtils::proxy = false;
std::string XmlUtils::proxy_host=""; //host:port format
std::string XmlUtils::proxy_user="";
std::string XmlUtils::proxy_pass="";

int
XmlUtils::parseInt (std::string s, int radix)
{
  int len = s.size ();
  int value = 0;
  if (s.empty ())
    return -1;
  for (int i = 0; i < len; i++) {
    if (radix == 10) {
      if (s[i] <= '9' && s[i] >= 0)
	value = (i == 0) ? (s[i] - '0') : radix * value + (s[i] - '0');

      else
	throw std::exception ();
    }
    else if (radix == 16) {
      //assumes that the encoding format has arranges the alphabets and numbers in increasing order
      if (s[i] <= '9' && s[i] >= 0)
	value = (i == 0) ? (s[i] - '0') : radix * value + (s[i] - '0');

      else if (s[i] <= 'F' && s[i] >= 'A')
	value =
	  (i ==
	   0) ? (s[i] - 'A') + 10 : radix * value + (s[i] - 'A') + 10;

      else if (s[i] <= 'f' && s[i] >= 'a')
	value =(i ==0) ? (s[i] - 'a') + 10 : radix * value + (s[i] - 'a') + 10;
    }
  }
  return value;
}


std::ostream &
XmlUtils::dbsp (std::ostream & str)
{
  return str << "  ";
}


std::ostream &
XmlUtils::blk (std::ostream & str)
{
  return str << std::endl << "*************" << std::endl;
}

/*
 * Fetch a document at the given URI  and store it in a file
 */

bool
XmlUtils::fetchUri(std::string uri,
		   std::string& filename)
{
  if(uri.find("http://")!=std::string::npos || 
     uri.find("ftp://") !=std::string::npos)
    {  
#ifdef WITH_CURL
      CURL * curl;
      CURLcode res;
      curl=curl_easy_init();
      FILE * file;
      filename=uri.substr(uri.rfind('/')+1);
      if(curl){
        file=fopen(filename.c_str(),"w");
        curl_easy_setopt(curl, CURLOPT_URL,uri.c_str());
        curl_easy_setopt(curl,CURLOPT_FILE,(void*)file);
	
	if (XmlUtils::proxy){
	  curl_easy_setopt(curl,CURLOPT_PROXY,XmlUtils::proxy_host.c_str());
	  std::string tmp=XmlUtils::proxy_user+":"+XmlUtils::proxy_pass;
	  curl_easy_setopt(curl,CURLOPT_PROXYUSERPWD,tmp.c_str());
	}
        res = curl_easy_perform(curl);

        curl_easy_cleanup(curl);
        fclose(file);
        if(res)
          return false;
        else 
          return true;
      }
#endif
      return false;
    }else 
      { 
        /*
         * assume its a file on the disk
         */
        filename=uri;
        std::ifstream ifs;
        ifs.open(filename.c_str(),std::ios::in);
        if(ifs.fail()) {
          ifs.close();
          return false;
        }
        else {
          ifs.close();
          return true;
        }

      }
}


std::string
XmlUtils::acceptSecretKey(const std::string& field)
{
  std::cerr<<field<<": ";
  char password [50];
#ifndef _WIN32  
  tcflag_t oflags;
  struct termios term;
  tcgetattr(STDIN_FILENO, &term);
  oflags = term.c_lflag;
  term.c_lflag = oflags & ~(ECHO | ECHOK | ICANON);
  term.c_cc[VTIME] = 1;
  tcsetattr(STDIN_FILENO, TCSANOW, &term);

  scanf("%s", password);

  term.c_lflag = oflags;
  term.c_cc[VTIME] = 0;
  tcsetattr(STDIN_FILENO, TCSANOW, &term);
#else
  scanf("%s", password);
#endif
  return password;
}
